@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <string.h>		/* For memset() */
+
 #include "gdk.h"
 #include "gdkinternals.h"
 
@@ -256,24 +258,87 @@ gdk_event_put (GdkEvent *event)
 }
 
 static GMemChunk *event_chunk = NULL;
+static GHashTable *event_hash = NULL;
 
 GdkEvent*
-_gdk_event_new (void)
+gdk_event_new (GdkEventType type)
 {
-  GdkEventPrivate *new_event;
+  GdkEventPrivate *new_private;
+  GdkEvent *new_event;
   
   if (event_chunk == NULL)
-    event_chunk = g_mem_chunk_new ("events",
-				   sizeof (GdkEventPrivate),
-				   4096,
-				   G_ALLOC_AND_FREE);
+    {
+      event_chunk = g_mem_chunk_new ("events",
+				     sizeof (GdkEventPrivate),
+				     4096,
+				     G_ALLOC_AND_FREE);
+      event_hash = g_hash_table_new (g_direct_hash, NULL);
+    }
   
-  new_event = g_chunk_new (GdkEventPrivate, event_chunk);
-  new_event->flags = 0;
+  new_private = g_chunk_new (GdkEventPrivate, event_chunk);
+  memset (new_private, 0, sizeof (GdkEventPrivate));
   
-  return (GdkEvent*) new_event;
+  new_private->flags = 0;
+  new_private->screen = NULL;
+
+  g_hash_table_insert (event_hash, new_private, GUINT_TO_POINTER (1));
+
+  new_event = (GdkEvent *) new_private;
+
+  new_event->any.type = type;
+
+  /*
+   * Bytewise 0 initialization is reasonable for most of the 
+   * current event types. Explicitely initialize double fields
+   * since I trust bytewise 0 == 0. less than for integers
+   * or pointers.
+   */
+  switch (type)
+    {
+    case GDK_MOTION_NOTIFY:
+      new_event->motion.x = 0.;
+      new_event->motion.y = 0.;
+      new_event->motion.x_root = 0.;
+      new_event->motion.y_root = 0.;
+      break;
+    case GDK_BUTTON_PRESS:
+    case GDK_2BUTTON_PRESS:
+    case GDK_3BUTTON_PRESS:
+    case GDK_BUTTON_RELEASE:
+      new_event->button.x = 0.;
+      new_event->button.y = 0.;
+      new_event->button.x_root = 0.;
+      new_event->button.y_root = 0.;
+      break;
+    case GDK_SCROLL:
+      new_event->scroll.x = 0.;
+      new_event->scroll.y = 0.;
+      new_event->scroll.x_root = 0.;
+      new_event->scroll.y_root = 0.;
+      break;
+    case GDK_ENTER_NOTIFY:
+    case GDK_LEAVE_NOTIFY:
+      new_event->crossing.x = 0.;
+      new_event->crossing.y = 0.;
+      new_event->crossing.x_root = 0.;
+      new_event->crossing.y_root = 0.;
+      break;
+    default:
+      break;
+    }
+  
+  return new_event;
 }
 
+static gboolean
+gdk_event_is_allocated (GdkEvent *event)
+{
+  if (event_hash)
+    return g_hash_table_lookup (event_hash, event) != NULL;
+
+  return FALSE;
+}
+ 
 /**
  * gdk_event_copy:
  * @event: a #GdkEvent
@@ -287,15 +352,22 @@ _gdk_event_new (void)
 GdkEvent*
 gdk_event_copy (GdkEvent *event)
 {
+  GdkEventPrivate *private;
+  GdkEventPrivate *new_private;
   GdkEvent *new_event;
   
   g_return_val_if_fail (event != NULL, NULL);
   
-  new_event = _gdk_event_new ();
-  
+  new_event = gdk_event_new (GDK_NOTHING);
+  new_private = (GdkEventPrivate *)new_event;
+
+  private = (GdkEventPrivate *)event;
+
   *new_event = *event;
   if (new_event->any.window)
     g_object_ref (new_event->any.window);
+
+  new_private->screen = private->screen;
   
   switch (event->any.type)
     {
@@ -399,7 +471,8 @@ gdk_event_free (GdkEvent *event)
     default:
       break;
     }
-  
+
+  g_hash_table_remove (event_hash, event);
   g_mem_chunk_free (event_chunk, event);
 }
 
@@ -738,6 +811,59 @@ gdk_event_get_axis (GdkEvent   *event,
   return gdk_device_get_axis (device, axes, axis_use, value);
 }
 
+/**
+ * gdk_event_set_screen:
+ * @event: a #GdkEvent
+ * @screen: a #GdkScreen
+ * 
+ * Sets the screen for @event to @screen. The event must
+ * have been allocated by GTK+, for instance, by
+ * gdk_event_copy().
+ **/
+void
+gdk_event_set_screen (GdkEvent  *event,
+		      GdkScreen *screen)
+{
+  GdkEventPrivate *private;
+  
+  g_return_if_fail (gdk_event_is_allocated (event));
+
+  private = (GdkEventPrivate *)event;
+  
+  private->screen = screen;
+}
+
+/**
+ * gdk_event_get_screen:
+ * @event: a #GdkEvent
+ * 
+ * Returns the screen for the event. The screen is
+ * typically the screen for event->any.window, but
+ * for events such as mouse events, it is the screen
+ * where the the pointer was when the event occurs -
+ * that is, the screen which has the root window 
+ * to which event->motion.x_root and
+ * event->motion.y_root are relative.
+ * 
+ * Return value: the screen for the event
+ **/
+GdkScreen *
+gdk_event_get_screen (GdkEvent *event)
+{
+  if (gdk_event_is_allocated (event))
+    {
+      GdkEventPrivate *private = (GdkEventPrivate *)event;
+
+      if (private->screen)
+	return private->screen;
+    }
+
+  if (event->any.window)
+    return gdk_drawable_get_screen (event->any.window);
+
+  return NULL;
+}
+
 /**
  * gdk_set_show_events:
  * @show_events:  %TRUE to output event debugging information.
@@ -927,10 +1053,10 @@ gdk_synthesize_window_state (GdkWindow     *window,
   
   old = ((GdkWindowObject*) temp_event.window_state.window)->state;
   
-  temp_event.window_state.changed_mask = (unset_flags | set_flags) ^ old;
   temp_event.window_state.new_window_state = old;
   temp_event.window_state.new_window_state |= set_flags;
   temp_event.window_state.new_window_state &= ~unset_flags;
+  temp_event.window_state.changed_mask = temp_event.window_state.new_window_state ^ old;
 
   if (temp_event.window_state.new_window_state == old)
     return; /* No actual work to do, nothing changed. */
