@@ -39,7 +39,6 @@
 #include "gtklistitem.h"
 #include "gtkscrolledwindow.h"
 #include "gtkmain.h"
-#include "gtksignal.h"
 #include "gtkwindow.h"
 #include "gdk/gdkkeysyms.h"
 #include "gtkcombo.h"
@@ -62,6 +61,8 @@ enum {
 
 static void         gtk_combo_class_init         (GtkComboClass    *klass);
 static void         gtk_combo_init               (GtkCombo         *combo);
+static void         gtk_combo_realize		 (GtkWidget	   *widget);
+static void         gtk_combo_unrealize		 (GtkWidget	   *widget);
 static void         gtk_combo_destroy            (GtkObject        *combo);
 static GtkListItem *gtk_combo_find               (GtkCombo         *combo);
 static gchar *      gtk_combo_func               (GtkListItem      *li);
@@ -105,7 +106,6 @@ static gint         gtk_combo_entry_key_press    (GtkEntry         *widget,
 static gint         gtk_combo_window_key_press   (GtkWidget        *window,
 						  GdkEventKey      *event,
 						  GtkCombo         *combo);
-static void         gtk_combo_item_destroy       (GtkObject        *object);
 static void         gtk_combo_size_allocate      (GtkWidget        *widget,
 						  GtkAllocation   *allocation);
 static void         gtk_combo_set_property       (GObject         *object,
@@ -126,10 +126,11 @@ gtk_combo_class_init (GtkComboClass * klass)
   GtkWidgetClass *widget_class;
 
   gobject_class = (GObjectClass *) klass;
-  parent_class = gtk_type_class (GTK_TYPE_HBOX);
   oclass = (GtkObjectClass *) klass;
   widget_class = (GtkWidgetClass *) klass;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gobject_class->set_property = gtk_combo_set_property; 
   gobject_class->get_property = gtk_combo_get_property; 
 
@@ -175,6 +176,8 @@ gtk_combo_class_init (GtkComboClass * klass)
   oclass->destroy = gtk_combo_destroy;
   
   widget_class->size_allocate = gtk_combo_size_allocate;
+  widget_class->realize = gtk_combo_realize;
+  widget_class->unrealize = gtk_combo_unrealize;
 }
 
 static void
@@ -185,7 +188,7 @@ gtk_combo_destroy (GtkObject *object)
   if (combo->popwin)
     {
       gtk_widget_destroy (combo->popwin);
-      gtk_widget_unref (combo->popwin);
+      g_object_unref (combo->popwin);
       combo->popwin = NULL;
     }
 
@@ -203,39 +206,39 @@ gtk_combo_entry_key_press (GtkEntry * entry, GdkEventKey * event, GtkCombo * com
       (event->state & GDK_MOD1_MASK)) 
     {
       GtkEditable *editable = GTK_EDITABLE (entry);
-    GCompletion * cmpl;
-    gchar* prefix;
-    gchar* nprefix = NULL;
-    gint pos;
+      GCompletion * cmpl;
+      gchar* prefix;
+      gchar* nprefix = NULL;
+      gint pos;
 
-    if ( !GTK_LIST (combo->list)->children )
-      return FALSE;
+      if ( !GTK_LIST (combo->list)->children )
+	return FALSE;
     
-    gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "key_press_event");
+      g_signal_stop_emission_by_name (entry, "key_press_event");
 
-    cmpl = g_completion_new ((GCompletionFunc)gtk_combo_func);
-    g_completion_add_items (cmpl, GTK_LIST (combo->list)->children);
+      cmpl = g_completion_new ((GCompletionFunc)gtk_combo_func);
+      g_completion_add_items (cmpl, GTK_LIST (combo->list)->children);
 
-    pos = gtk_editable_get_position (editable);
-    prefix = gtk_editable_get_chars (editable, 0, pos);
+      pos = gtk_editable_get_position (editable);
+      prefix = gtk_editable_get_chars (editable, 0, pos);
 
-    g_completion_complete (cmpl, prefix, &nprefix);
+      g_completion_complete (cmpl, prefix, &nprefix);
 
-    if (nprefix && strlen (nprefix) > strlen (prefix)) 
-      {
-    	gtk_editable_insert_text (editable, nprefix + pos, 
-				  strlen (nprefix) - strlen (prefix), &pos);
-	gtk_editable_set_position (editable, pos);
+      if (nprefix && strlen (nprefix) > strlen (prefix)) 
+	{
+	  gtk_editable_insert_text (editable, nprefix + pos, 
+				    strlen (nprefix) - strlen (prefix), &pos);
+	  gtk_editable_set_position (editable, pos);
+	}
+
+      if (nprefix)
+	g_free (nprefix);
+      g_free (prefix);
+      g_completion_free (cmpl);
+
+      return TRUE;
     }
 
-    if (nprefix)
-      g_free (nprefix);
-    g_free (prefix);
-    g_completion_free (cmpl);
-
-    return TRUE;
-  }
-
   if (!combo->use_arrows || !GTK_LIST (combo->list)->children)
     return FALSE;
 
@@ -254,7 +257,7 @@ gtk_combo_entry_key_press (GtkEntry * entry, GdkEventKey * event, GtkCombo * com
       if (li)
 	{
 	  gtk_list_select_child (GTK_LIST (combo->list), GTK_WIDGET (li->data));
-	  gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "key_press_event");
+	  g_signal_stop_emission_by_name (entry, "key_press_event");
 	  return TRUE;
 	}
     }
@@ -271,7 +274,7 @@ gtk_combo_entry_key_press (GtkEntry * entry, GdkEventKey * event, GtkCombo * com
       if (li)
 	{
 	  gtk_list_select_child (GTK_LIST (combo->list), GTK_WIDGET (li->data));
-	  gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "key_press_event");
+	  g_signal_stop_emission_by_name (entry, "key_press_event");
 	  return TRUE;
 	}
     }
@@ -292,11 +295,12 @@ gtk_combo_window_key_press (GtkWidget   *window,
 	  if (GTK_WIDGET_HAS_GRAB (combo->popwin))
 	    {
 	      gtk_grab_remove (combo->popwin);
-	      gdk_pointer_ungrab (event->time);
+	      gdk_display_pointer_ungrab (gtk_widget_get_display (window),
+					  event->time);
 	    }
 	}
 
-      gtk_signal_emit_stop_by_name (GTK_OBJECT (window), "key_press_event");
+      g_signal_stop_emission_by_name (window, "key_press_event");
 
       return TRUE;
     }
@@ -308,29 +312,39 @@ static GtkListItem *
 gtk_combo_find (GtkCombo * combo)
 {
   const gchar *text;
+  GtkListItem *found = NULL;
   gchar *ltext;
+  gchar *compare_text;
   GList *clist;
-  int (*string_compare) (const char *, const char *);
-
-  if (combo->case_sensitive)
-    string_compare = strcmp;
-  else
-    string_compare = g_strcasecmp;
 
   text = gtk_entry_get_text (GTK_ENTRY (combo->entry));
-  clist = GTK_LIST (combo->list)->children;
-
-  while (clist && clist->data)
+  if (combo->case_sensitive)
+    compare_text = (gchar *)text;
+  else
+    compare_text = g_utf8_casefold (text, -1);
+  
+  for (clist = GTK_LIST (combo->list)->children;
+       !found && clist;
+       clist = clist->next)
     {
       ltext = gtk_combo_func (GTK_LIST_ITEM (clist->data));
       if (!ltext)
 	continue;
-      if (!(*string_compare) (ltext, text))
-	return (GtkListItem *) clist->data;
-      clist = clist->next;
+
+      if (!combo->case_sensitive)
+	ltext = g_utf8_casefold (ltext, -1);
+
+      if (strcmp (ltext, compare_text) == 0)
+	found = clist->data;
+
+      if (!combo->case_sensitive)
+	g_free (ltext);
     }
 
-  return NULL;
+  if (!combo->case_sensitive)
+    g_free (compare_text);
+
+  return found;
 }
 
 static gchar *
@@ -339,13 +353,13 @@ gtk_combo_func (GtkListItem * li)
   GtkWidget *label;
   gchar *ltext = NULL;
 
-  ltext = (gchar *) gtk_object_get_data (GTK_OBJECT (li), gtk_combo_string_key);
+  ltext = g_object_get_data (G_OBJECT (li), gtk_combo_string_key);
   if (!ltext)
     {
       label = GTK_BIN (li)->child;
       if (!label || !GTK_IS_LABEL (label))
 	return NULL;
-      gtk_label_get (GTK_LABEL (label), &ltext);
+      ltext = (gchar *) gtk_label_get_text (GTK_LABEL (label));
     }
   return ltext;
 }
@@ -368,6 +382,8 @@ gtk_combo_entry_focus_out (GtkEntry * entry, GdkEventFocus * event, GtkCombo * c
 
   if (combo->value_in_list && !gtk_combo_find (combo))
     {
+      GSource *focus_idle;
+      
       /* gdk_beep(); *//* this can be annoying */
       if (combo->ok_if_empty && !strcmp (gtk_entry_get_text (entry), ""))
 	return FALSE;
@@ -378,10 +394,15 @@ gtk_combo_entry_focus_out (GtkEntry * entry, GdkEventFocus * event, GtkCombo * c
       /* this is needed because if we call gtk_widget_grab_focus() 
          it isn't guaranteed it's the *last* call before the main-loop,
          so the focus can be lost anyway...
-         the signal_emit_stop doesn't seem to work either...
+         the signal_stop_emission doesn't seem to work either...
        */
-      gtk_idle_add ((GtkFunction) gtk_combo_focus_idle, combo);
-      /*gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "focus_out_event"); */
+      focus_idle = g_idle_source_new ();
+      g_source_set_closure (focus_idle,
+			    g_cclosure_new_object (G_CALLBACK (gtk_combo_focus_idle),
+						   G_OBJECT (combo)));
+      g_source_attach (focus_idle, NULL);
+      
+      /*g_signal_stop_emission_by_name (entry, "focus_out_event"); */
       return TRUE;
     }
   return FALSE;
@@ -404,16 +425,19 @@ gtk_combo_get_pos (GtkCombo * combo, gint * x, gint * y, gint * height, gint * w
   gint work_height;
   gint old_height;
   gint old_width;
+  gint scrollbar_spacing;
   
   widget = GTK_WIDGET (combo);
   popup  = GTK_SCROLLED_WINDOW (combo->popup);
   popwin = GTK_BIN (combo->popwin);
-  
+
+  scrollbar_spacing = _gtk_scrolled_window_get_scrollbar_spacing (popup);
+
   gdk_window_get_origin (combo->entry->window, x, y);
   real_height = MIN (combo->entry->requisition.height, 
 		     combo->entry->allocation.height);
   *y += real_height;
-  avail_height = gdk_screen_height () - *y;
+  avail_height = gdk_screen_get_height (gtk_widget_get_screen (widget)) - *y;
   
   gtk_widget_size_request (combo->list, &list_requisition);
   min_height = MIN (list_requisition.height, 
@@ -432,7 +456,7 @@ gtk_combo_get_pos (GtkCombo * combo, gint * x, gint * y, gint * height, gint * w
 		 2 * GTK_CONTAINER (popwin->child)->border_width +
 		 2 * GTK_CONTAINER (combo->popup)->border_width +
 		 2 * GTK_CONTAINER (GTK_BIN (popup)->child)->border_width +
-		 2 * GTK_BIN (popup)->child->style->xthickness);
+		 2 * GTK_BIN (popup)->child->style->ythickness);
   
   do 
     {
@@ -442,21 +466,26 @@ gtk_combo_get_pos (GtkCombo * combo, gint * x, gint * y, gint * height, gint * w
       if (!show_hscroll &&
 	  alloc_width < list_requisition.width)
 	{
-	  work_height += (popup->hscrollbar->requisition.height +
-			  GTK_SCROLLED_WINDOW_GET_CLASS (combo->popup)->scrollbar_spacing);
+	  GtkRequisition requisition;
+	  
+	  gtk_widget_size_request (popup->hscrollbar, &requisition);
+	  work_height += (requisition.height + scrollbar_spacing);
+	  
 	  show_hscroll = TRUE;
 	}
       if (!show_vscroll && 
 	  work_height + list_requisition.height > avail_height)
 	{
+	  GtkRequisition requisition;
+	  
 	  if (work_height + min_height > avail_height && 
 	      *y - real_height > avail_height)
 	    {
 	      *y -= (work_height + list_requisition.height + real_height);
 	      break;
 	    }
-	  alloc_width -= (popup->vscrollbar->requisition.width +
-			  GTK_SCROLLED_WINDOW_GET_CLASS (combo->popup)->scrollbar_spacing);
+	  gtk_widget_size_request (popup->hscrollbar, &requisition);
+	  alloc_width -= (requisition.width + scrollbar_spacing);
 	  show_vscroll = TRUE;
 	}
     } while (old_width != alloc_width || old_height != work_height);
@@ -490,7 +519,7 @@ gtk_combo_popup_list (GtkCombo * combo)
     }
 
   gtk_window_move (GTK_WINDOW (combo->popwin), x, y);
-  gtk_widget_set_usize (combo->popwin, width, height);
+  gtk_widget_set_size_request (combo->popwin, width, height);
   gtk_widget_show (combo->popwin);
 
   gtk_widget_grab_focus (combo->popwin);
@@ -510,7 +539,8 @@ gtk_combo_popdown_list (GtkCombo *combo)
   if (GTK_WIDGET_HAS_GRAB (combo->popwin))
     {
       gtk_grab_remove (combo->popwin);
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
+      gdk_display_pointer_ungrab (gtk_widget_get_display (GTK_WIDGET (combo)),
+				  GDK_CURRENT_TIME);
     }
   
   gtk_widget_hide (combo->popwin);
@@ -579,7 +609,7 @@ gtk_combo_update_entry (GtkList * list, GtkCombo * combo)
   char *text;
 
   gtk_grab_remove (GTK_WIDGET (combo));
-  gtk_signal_handler_block (GTK_OBJECT (list), combo->list_change_id);
+  g_signal_handler_block (list, combo->list_change_id);
   if (list->selection)
     {
       text = gtk_combo_func (GTK_LIST_ITEM (list->selection->data));
@@ -587,7 +617,7 @@ gtk_combo_update_entry (GtkList * list, GtkCombo * combo)
 	text = "";
       gtk_entry_set_text (GTK_ENTRY (combo->entry), text);
     }
-  gtk_signal_handler_unblock (GTK_OBJECT (list), combo->list_change_id);
+  g_signal_handler_unblock (list, combo->list_change_id);
 }
 
 static void
@@ -599,13 +629,13 @@ gtk_combo_update_list (GtkEntry * entry, GtkCombo * combo)
 
   gtk_grab_remove (GTK_WIDGET (combo));
 
-  gtk_signal_handler_block (GTK_OBJECT (entry), combo->entry_change_id);
+  g_signal_handler_block (entry, combo->entry_change_id);
   if (slist && slist->data)
     gtk_list_unselect_child (list, GTK_WIDGET (slist->data));
   li = gtk_combo_find (combo);
   if (li)
     gtk_list_select_child (list, GTK_WIDGET (li));
-  gtk_signal_handler_unblock (GTK_OBJECT (entry), combo->entry_change_id);
+  g_signal_handler_unblock (entry, combo->entry_change_id);
 }
 
 static gint
@@ -687,7 +717,7 @@ gtk_combo_list_enter (GtkWidget        *widget,
       (combo->current_button != 0) && 
       (!GTK_WIDGET_HAS_GRAB (combo->list)))
     {
-      GdkEvent tmp_event;
+      GdkEvent *tmp_event = gdk_event_new (GDK_BUTTON_PRESS);
       gint x, y;
       GdkModifierType mask;
 
@@ -698,19 +728,19 @@ gtk_combo_list_enter (GtkWidget        *widget,
        */
       gdk_window_get_pointer (combo->list->window, &x, &y, &mask);
 
-      tmp_event.button.type = GDK_BUTTON_PRESS;
-      tmp_event.button.window = combo->list->window;
-      tmp_event.button.send_event = TRUE;
-      tmp_event.button.time = GDK_CURRENT_TIME; /* bad */
-      tmp_event.button.x = x;
-      tmp_event.button.y = y;
+      tmp_event->button.window = g_object_ref (combo->list->window);
+      tmp_event->button.send_event = TRUE;
+      tmp_event->button.time = GDK_CURRENT_TIME; /* bad */
+      tmp_event->button.x = x;
+      tmp_event->button.y = y;
       /* We leave all the XInput fields unfilled here, in the expectation
        * that GtkList doesn't care.
        */
-      tmp_event.button.button = combo->current_button;
-      tmp_event.button.state = mask;
+      tmp_event->button.button = combo->current_button;
+      tmp_event->button.state = mask;
 
-      gtk_widget_event (combo->list, &tmp_event);
+      gtk_widget_event (combo->list, tmp_event);
+      gdk_event_free (tmp_event);
     }
 
   return FALSE;
@@ -731,13 +761,21 @@ gtk_combo_list_key_press (GtkWidget * widget, GdkEventKey * event, GtkCombo * co
   return FALSE;
 }
 
+static void
+combo_event_box_realize (GtkWidget *widget)
+{
+  GdkCursor *cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+						  GDK_TOP_LEFT_ARROW);
+  gdk_window_set_cursor (widget->window, cursor);
+  gdk_cursor_unref (cursor);
+}
+
 static void
 gtk_combo_init (GtkCombo * combo)
 {
   GtkWidget *arrow;
   GtkWidget *frame;
   GtkWidget *event_box;
-  GdkCursor *cursor;
 
   combo->case_sensitive = FALSE;
   combo->value_in_list = FALSE;
@@ -755,36 +793,36 @@ gtk_combo_init (GtkCombo * combo)
   GTK_WIDGET_UNSET_FLAGS (combo->button, GTK_CAN_FOCUS);
   gtk_widget_show (combo->entry);
   gtk_widget_show (combo->button);
-  combo->entry_change_id = gtk_signal_connect (GTK_OBJECT (combo->entry), "changed",
-			      (GtkSignalFunc) gtk_combo_update_list, combo);
-  gtk_signal_connect (GTK_OBJECT (combo->entry), "key_press_event",
-		      (GtkSignalFunc) gtk_combo_entry_key_press, combo);
-  gtk_signal_connect_after (GTK_OBJECT (combo->entry), "focus_out_event",
-			    (GtkSignalFunc) gtk_combo_entry_focus_out, combo);
-  combo->activate_id = gtk_signal_connect (GTK_OBJECT (combo->entry), "activate",
-		      (GtkSignalFunc) gtk_combo_activate, combo);
-  gtk_signal_connect (GTK_OBJECT (combo->button), "button_press_event",
-		      (GtkSignalFunc) gtk_combo_popup_button_press, combo);
-  gtk_signal_connect (GTK_OBJECT (combo->button), "leave_notify_event",
-		      (GtkSignalFunc) gtk_combo_popup_button_leave, combo);
+  combo->entry_change_id = g_signal_connect (combo->entry, "changed",
+					     G_CALLBACK (gtk_combo_update_list),
+					     combo);
+  g_signal_connect (combo->entry, "key_press_event",
+		    G_CALLBACK (gtk_combo_entry_key_press), combo);
+  g_signal_connect_after (combo->entry, "focus_out_event",
+			  G_CALLBACK (gtk_combo_entry_focus_out), combo);
+  combo->activate_id = g_signal_connect (combo->entry, "activate",
+					 G_CALLBACK (gtk_combo_activate),
+					 combo);
+  g_signal_connect (combo->button, "button_press_event",
+		    G_CALLBACK (gtk_combo_popup_button_press), combo);
+  g_signal_connect (combo->button, "leave_notify_event",
+		    G_CALLBACK (gtk_combo_popup_button_leave), combo);
 
   combo->popwin = gtk_window_new (GTK_WINDOW_POPUP);
-  gtk_widget_ref (combo->popwin);
+  g_object_ref (combo->popwin);
   gtk_window_set_resizable (GTK_WINDOW (combo->popwin), FALSE);
 
-  gtk_signal_connect (GTK_OBJECT (combo->popwin), "key_press_event",
-		      GTK_SIGNAL_FUNC (gtk_combo_window_key_press), combo);
+  g_signal_connect (combo->popwin, "key_press_event",
+		    G_CALLBACK (gtk_combo_window_key_press), combo);
   
   gtk_widget_set_events (combo->popwin, GDK_KEY_PRESS_MASK);
 
   event_box = gtk_event_box_new ();
   gtk_container_add (GTK_CONTAINER (combo->popwin), event_box);
+  g_signal_connect (event_box, "realize",
+		    G_CALLBACK (combo_event_box_realize), NULL);
   gtk_widget_show (event_box);
 
-  gtk_widget_realize (event_box);
-  cursor = gdk_cursor_new (GDK_TOP_LEFT_ARROW);
-  gdk_window_set_cursor (event_box->window, cursor);
-  gdk_cursor_destroy (cursor);
 
   frame = gtk_frame_new (NULL);
   gtk_container_add (GTK_CONTAINER (event_box), frame);
@@ -813,50 +851,79 @@ gtk_combo_init (GtkCombo * combo)
 				       gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (combo->popup)));
   gtk_widget_show (combo->list);
 
-  combo->list_change_id = gtk_signal_connect (GTK_OBJECT (combo->list), "selection_changed",
-			     (GtkSignalFunc) gtk_combo_update_entry, combo);
-  gtk_signal_connect (GTK_OBJECT (combo->popwin), "key_press_event",
-		      (GtkSignalFunc) gtk_combo_list_key_press, combo);
-  gtk_signal_connect (GTK_OBJECT (combo->popwin), "button_press_event",
-		      GTK_SIGNAL_FUNC (gtk_combo_button_press), combo);
+  combo->list_change_id = g_signal_connect (combo->list,
+					    "selection_changed",
+					    G_CALLBACK (gtk_combo_update_entry),
+					    combo);
+  g_signal_connect (combo->popwin, "key_press_event",
+		    G_CALLBACK (gtk_combo_list_key_press), combo);
+  g_signal_connect (combo->popwin, "button_press_event",
+		    G_CALLBACK (gtk_combo_button_press), combo);
+
+  g_signal_connect (combo->list, "event_after",
+		    G_CALLBACK (gtk_combo_button_event_after), combo);
 
-  gtk_signal_connect (GTK_OBJECT (combo->list), "event_after",
-		      (GtkSignalFunc) gtk_combo_button_event_after, combo);
   /* We connect here on the button, because we'll have a grab on it
    * when the event occurs. But we are actually interested in enters
    * for the combo->list.
    */
-  gtk_signal_connect (GTK_OBJECT (combo->button), "enter_notify_event",
-		      GTK_SIGNAL_FUNC (gtk_combo_list_enter), combo);
+  g_signal_connect (combo->button, "enter_notify_event",
+		    G_CALLBACK (gtk_combo_list_enter), combo);
 }
 
-GtkType
+static void
+gtk_combo_realize (GtkWidget *widget)
+{
+  GtkCombo *combo = GTK_COMBO (widget);
+
+  gtk_window_set_screen (GTK_WINDOW (combo->popwin), 
+			 gtk_widget_get_screen (widget));
+  
+  GTK_WIDGET_CLASS( parent_class )->realize (widget);  
+}
+
+static void        
+gtk_combo_unrealize (GtkWidget *widget)
+{
+  GtkCombo *combo = GTK_COMBO (widget);
+
+  gtk_combo_popdown_list (combo);
+  gtk_widget_unrealize (combo->popwin);
+  
+  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+}
+
+GType
 gtk_combo_get_type (void)
 {
-  static GtkType combo_type = 0;
+  static GType combo_type = 0;
 
   if (!combo_type)
     {
-      static const GtkTypeInfo combo_info =
+      static const GTypeInfo combo_info =
       {
-	"GtkCombo",
-	sizeof (GtkCombo),
 	sizeof (GtkComboClass),
-	(GtkClassInitFunc) gtk_combo_class_init,
-	(GtkObjectInitFunc) gtk_combo_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_combo_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkCombo),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_combo_init,
       };
-      combo_type = gtk_type_unique (GTK_TYPE_HBOX, &combo_info);
+
+      combo_type = g_type_register_static (GTK_TYPE_HBOX, "GtkCombo",
+					   &combo_info, 0);
     }
+
   return combo_type;
 }
 
 GtkWidget*
 gtk_combo_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (GTK_TYPE_COMBO));
+  return g_object_new (GTK_TYPE_COMBO, NULL);
 }
 
 void
@@ -936,6 +1003,8 @@ gtk_combo_set_popdown_strings (GtkCombo * combo, GList * strings)
   g_return_if_fail (GTK_IS_COMBO (combo));
   g_return_if_fail (strings != NULL);
 
+  gtk_combo_popdown_list (combo);
+
   gtk_list_clear_items (GTK_LIST (combo->list), 0, -1);
   list = strings;
   while (list)
@@ -947,48 +1016,14 @@ gtk_combo_set_popdown_strings (GtkCombo * combo, GList * strings)
     }
 }
 
-static void
-gtk_combo_item_destroy (GtkObject * object)
-{
-  gchar *key;
-
-  key = gtk_object_get_data (object, gtk_combo_string_key);
-  if (key)
-    {
-      gtk_object_remove_data (object, gtk_combo_string_key);
-      g_free (key);
-    }
-}
-
 void
 gtk_combo_set_item_string (GtkCombo * combo, GtkItem * item, const gchar * item_value)
 {
-  gchar *val;
-  gint connected = 0;
-
   g_return_if_fail (GTK_IS_COMBO (combo));
   g_return_if_fail (item != NULL);
 
-  val = gtk_object_get_data (GTK_OBJECT (item), gtk_combo_string_key);
-  if (val) 
-    {
-      g_free (val);
-      connected = 1;
-    }
-  if (item_value)
-    {
-      val = g_strdup (item_value);
-      gtk_object_set_data (GTK_OBJECT (item), gtk_combo_string_key, val);
-      if (!connected)
-        gtk_signal_connect (GTK_OBJECT (item), "destroy",
-			  (GtkSignalFunc) gtk_combo_item_destroy, val);
-    }
-  else 
-    {
-      gtk_object_set_data (GTK_OBJECT (item), gtk_combo_string_key, NULL);
-      if (connected)
-	gtk_signal_disconnect_by_data(GTK_OBJECT (item), val);
-    }
+  g_object_set_data_full (G_OBJECT (item), gtk_combo_string_key,
+			  g_strdup (item_value), g_free);
 }
 
 static void
@@ -1023,7 +1058,7 @@ gtk_combo_disable_activate (GtkCombo* combo)
   g_return_if_fail (GTK_IS_COMBO (combo));
 
   if ( combo->activate_id ) {
-    gtk_signal_disconnect (GTK_OBJECT(combo->entry), combo->activate_id);
+    g_signal_handler_disconnect (combo->entry, combo->activate_id);
     combo->activate_id = 0;
   }
 }
