@@ -207,21 +207,21 @@ _gtk_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT,UINT,UINT (gtkmarshalers.list:28) */
+/* BOOLEAN:OBJECT,UINT,FLAGS (gtkmarshalers.list:28) */
 void
-_gtk_marshal_BOOLEAN__OBJECT_UINT_UINT (GClosure     *closure,
-                                        GValue       *return_value,
-                                        guint         n_param_values,
-                                        const GValue *param_values,
-                                        gpointer      invocation_hint,
-                                        gpointer      marshal_data)
+_gtk_marshal_BOOLEAN__OBJECT_UINT_FLAGS (GClosure     *closure,
+                                         GValue       *return_value,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint,
+                                         gpointer      marshal_data)
 {
-  typedef gboolean (*GMarshalFunc_BOOLEAN__OBJECT_UINT_UINT) (gpointer     data1,
-                                                              gpointer     arg_1,
-                                                              guint        arg_2,
-                                                              guint        arg_3,
-                                                              gpointer     data2);
-  register GMarshalFunc_BOOLEAN__OBJECT_UINT_UINT callback;
+  typedef gboolean (*GMarshalFunc_BOOLEAN__OBJECT_UINT_FLAGS) (gpointer     data1,
+                                                               gpointer     arg_1,
+                                                               guint        arg_2,
+                                                               guint        arg_3,
+                                                               gpointer     data2);
+  register GMarshalFunc_BOOLEAN__OBJECT_UINT_FLAGS callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
   gboolean v_return;
@@ -239,12 +239,12 @@ _gtk_marshal_BOOLEAN__OBJECT_UINT_UINT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_BOOLEAN__OBJECT_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_BOOLEAN__OBJECT_UINT_FLAGS) (marshal_data ? marshal_data : cc->callback);
 
   v_return = callback (data1,
                        g_marshal_value_peek_object (param_values + 1),
                        g_marshal_value_peek_uint (param_values + 2),
-                       g_marshal_value_peek_uint (param_values + 3),
+                       g_marshal_value_peek_flags (param_values + 3),
                        data2);
 
   g_value_set_boolean (return_value, v_return);
@@ -1871,7 +1871,46 @@ _gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
 
 /* VOID:STRING,INT,POINTER (gtkmarshalers.list:83) */
 
-/* VOID:UINT,UINT (gtkmarshalers.list:84) */
+/* VOID:UINT,FLAGS,BOXED (gtkmarshalers.list:84) */
+void
+_gtk_marshal_VOID__UINT_FLAGS_BOXED (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__UINT_FLAGS_BOXED) (gpointer     data1,
+                                                       guint        arg_1,
+                                                       guint        arg_2,
+                                                       gpointer     arg_3,
+                                                       gpointer     data2);
+  register GMarshalFunc_VOID__UINT_FLAGS_BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__UINT_FLAGS_BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_uint (param_values + 1),
+            g_marshal_value_peek_flags (param_values + 2),
+            g_marshal_value_peek_boxed (param_values + 3),
+            data2);
+}
+
+/* VOID:UINT,UINT (gtkmarshalers.list:85) */
 void
 _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
                               GValue       *return_value,
@@ -1908,45 +1947,6 @@ _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,UINT,BOXED (gtkmarshalers.list:85) */
-void
-_gtk_marshal_VOID__UINT_UINT_BOXED (GClosure     *closure,
-                                    GValue       *return_value,
-                                    guint         n_param_values,
-                                    const GValue *param_values,
-                                    gpointer      invocation_hint,
-                                    gpointer      marshal_data)
-{
-  typedef void (*GMarshalFunc_VOID__UINT_UINT_BOXED) (gpointer     data1,
-                                                      guint        arg_1,
-                                                      guint        arg_2,
-                                                      gpointer     arg_3,
-                                                      gpointer     data2);
-  register GMarshalFunc_VOID__UINT_UINT_BOXED callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-
-  g_return_if_fail (n_param_values == 4);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_VOID__UINT_UINT_BOXED) (marshal_data ? marshal_data : cc->callback);
-
-  callback (data1,
-            g_marshal_value_peek_uint (param_values + 1),
-            g_marshal_value_peek_uint (param_values + 2),
-            g_marshal_value_peek_boxed (param_values + 3),
-            data2);
-}
-
 /* VOID:UINT,STRING (gtkmarshalers.list:86) */
 void
 _gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
