@@ -34,7 +34,6 @@
 #include "gtkmenubar.h"
 #include "gtkmenuitem.h"
 #include "gtkseparatormenuitem.h"
-#include "gtksignal.h"
 
 
 #define BORDER_SPACING  3
@@ -103,10 +102,10 @@ static guint32	last_submenu_deselect_time = 0;
 
 
 
-GtkType
+GType
 gtk_menu_item_get_type (void)
 {
-  static GtkType menu_item_type = 0;
+  static GType menu_item_type = 0;
 
   if (!menu_item_type)
     {
@@ -123,7 +122,8 @@ gtk_menu_item_get_type (void)
 	(GInstanceInitFunc) gtk_menu_item_init,
       };
 
-      menu_item_type = g_type_register_static (GTK_TYPE_ITEM, "GtkMenuItem", &menu_item_info, 0);
+      menu_item_type = g_type_register_static (GTK_TYPE_ITEM, "GtkMenuItem",
+					       &menu_item_info, 0);
     }
 
   return menu_item_type;
@@ -169,39 +169,52 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
   klass->hide_on_activate = TRUE;
 
   menu_item_signals[ACTIVATE] =
-    gtk_signal_new ("activate",
-                    GTK_RUN_FIRST | GTK_RUN_ACTION,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, activate),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("activate",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, activate),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   widget_class->activate_signal = menu_item_signals[ACTIVATE];
 
   menu_item_signals[ACTIVATE_ITEM] =
-    gtk_signal_new ("activate_item",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, activate_item),
-                    gtk_signal_default_marshaller,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("activate_item",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, activate_item),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 
   menu_item_signals[TOGGLE_SIZE_REQUEST] =
-    gtk_signal_new ("toggle_size_request",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, toggle_size_request),
-                    _gtk_marshal_VOID__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_POINTER);
+    g_signal_new ("toggle_size_request",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, toggle_size_request),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__POINTER,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_POINTER);
 
   menu_item_signals[TOGGLE_SIZE_ALLOCATE] =
-    gtk_signal_new ("toggle_size_allocate",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, toggle_size_allocate),
-                    _gtk_marshal_NONE__INT,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_INT);
+    g_signal_new ("toggle_size_allocate",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, toggle_size_allocate),
+		  NULL, NULL,
+		  _gtk_marshal_NONE__INT,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_INT);
+
+  gtk_widget_class_install_style_property_parser (widget_class,
+						  g_param_spec_enum ("selected_shadow_type",
+								     "Selected Shadow Type",
+								     "Shadow type when item is selected",
+								     GTK_TYPE_SHADOW_TYPE,
+								     GTK_SHADOW_OUT,
+								     G_PARAM_READABLE),
+						  gtk_rc_property_parse_enum);
 }
 
 static void
@@ -223,7 +236,7 @@ gtk_menu_item_init (GtkMenuItem *menu_item)
 GtkWidget*
 gtk_menu_item_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (gtk_menu_item_get_type ()));
+  return g_object_new (GTK_TYPE_MENU_ITEM, NULL);
 }
 
 GtkWidget*
@@ -261,7 +274,7 @@ gtk_menu_item_new_with_mnemonic (const gchar *label)
   GtkWidget *accel_label;
 
   menu_item = gtk_menu_item_new ();
-  accel_label = gtk_type_new (GTK_TYPE_ACCEL_LABEL);
+  accel_label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (accel_label), label);
   gtk_misc_set_alignment (GTK_MISC (accel_label), 0.0, 0.5);
 
@@ -390,7 +403,7 @@ gtk_menu_item_activate (GtkMenuItem *menu_item)
 {
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
   
-  gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[ACTIVATE]);
+  g_signal_emit (menu_item, menu_item_signals[ACTIVATE], 0);
 }
 
 void
@@ -399,7 +412,7 @@ gtk_menu_item_toggle_size_request (GtkMenuItem *menu_item,
 {
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
-  gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[TOGGLE_SIZE_REQUEST], requisition);
+  g_signal_emit (menu_item, menu_item_signals[TOGGLE_SIZE_REQUEST], 0, requisition);
 }
 
 void
@@ -408,7 +421,7 @@ gtk_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
 {
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
-  gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[TOGGLE_SIZE_ALLOCATE], allocation);
+  g_signal_emit (menu_item, menu_item_signals[TOGGLE_SIZE_ALLOCATE], 0, allocation);
 }
 
 static void
@@ -531,7 +544,7 @@ gtk_menu_item_realize (GtkWidget *widget)
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
   widget->window = gtk_widget_get_parent_window (widget);
-  gdk_window_ref (widget->window);
+  g_object_ref (widget->window);
   
   attributes.x = widget->allocation.x;
   attributes.y = widget->allocation.y;
@@ -593,7 +606,7 @@ gtk_menu_item_paint (GtkWidget    *widget,
 {
   GtkMenuItem *menu_item;
   GtkStateType state_type;
-  GtkShadowType shadow_type;
+  GtkShadowType shadow_type, selected_shadow_type;
   gint width, height;
   gint x, y;
   gint border_width = GTK_CONTAINER (widget)->border_width;
@@ -611,13 +624,18 @@ gtk_menu_item_paint (GtkWidget    *widget,
       
       if ((state_type == GTK_STATE_PRELIGHT) &&
 	  (GTK_BIN (menu_item)->child))
-	gtk_paint_box (widget->style,
-		       widget->window,
-		       GTK_STATE_PRELIGHT,
-		       GTK_SHADOW_OUT,
-		       area, widget, "menuitem",
-		       x, y, width, height);
-
+	{
+	  gtk_widget_style_get (widget,
+				"selected_shadow_type", &selected_shadow_type,
+				NULL);
+	  gtk_paint_box (widget->style,
+			 widget->window,
+			 GTK_STATE_PRELIGHT,
+			 selected_shadow_type,
+			 area, widget, "menuitem",
+			 x, y, width, height);
+	}
+  
       if (menu_item->submenu && menu_item->show_submenu_indicator)
 	{
 	  GtkRequisition child_requisition;
@@ -710,7 +728,7 @@ gtk_real_menu_item_select (GtkItem *item)
     }
   
   gtk_widget_set_state (GTK_WIDGET (menu_item), GTK_STATE_PRELIGHT);
-  gtk_widget_draw (GTK_WIDGET (menu_item), NULL);
+  gtk_widget_queue_draw (GTK_WIDGET (menu_item));
 }
 
 static void
@@ -743,7 +761,7 @@ gtk_real_menu_item_deselect (GtkItem *item)
     }
 
   gtk_widget_set_state (GTK_WIDGET (menu_item), GTK_STATE_NORMAL);
-  gtk_widget_draw (GTK_WIDGET (menu_item), NULL);
+  gtk_widget_queue_draw (GTK_WIDGET (menu_item));
 }
 
 static gboolean
@@ -759,7 +777,7 @@ gtk_menu_item_mnemonic_activate (GtkWidget *widget,
 
     }
   else
-    gtk_signal_emit (GTK_OBJECT (widget), menu_item_signals[ACTIVATE_ITEM]);
+    g_signal_emit (widget, menu_item_signals[ACTIVATE_ITEM], 0);
   
   return TRUE;
 }
@@ -790,7 +808,7 @@ gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
 	  gtk_menu_shell_select_item (GTK_MENU_SHELL (widget->parent), widget); 
 	  gtk_menu_item_popup_submenu (widget); 
 
-	  _gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu));
+	  gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu), FALSE);
 	  submenu = GTK_MENU_SHELL (menu_item->submenu);
 	}
     }
@@ -867,6 +885,7 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   GtkMenuItem *menu_item;
   GtkWidget *widget;
   GtkWidget *parent_menu_item;
+  GdkScreen *screen;
   gint screen_width;
   gint screen_height;
   gint twidth, theight;
@@ -882,8 +901,9 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   twidth = GTK_WIDGET (menu)->requisition.width;
   theight = GTK_WIDGET (menu)->requisition.height;
 
-  screen_width = gdk_screen_width ();
-  screen_height = gdk_screen_height ();
+  screen = gtk_widget_get_screen (widget);
+  screen_width = gdk_screen_get_width (screen);
+  screen_height = gdk_screen_get_height (screen);
 
   if (!gdk_window_get_origin (widget->window, &tx, &ty))
     {
@@ -1088,7 +1108,7 @@ _gtk_menu_item_refresh_accel_path (GtkMenuItem   *menu_item,
       return;
     }
 
-  path = _gtk_widget_get_accel_path (widget);
+  path = _gtk_widget_get_accel_path (widget, NULL);
   if (!path)					/* no active accel_path yet */
     {
       path = menu_item->accel_path;
@@ -1113,7 +1133,8 @@ _gtk_menu_item_refresh_accel_path (GtkMenuItem   *menu_item,
 /**
  * gtk_menu_item_set_accel_path
  * @menu_item:  a valid #GtkMenuItem
- * @accel_path: accelerator path, corresponding to this menu item's functionality
+ * @accel_path: accelerator path, corresponding to this menu item's
+ *              functionality, or %NULL to unset the current path.
  *
  * Set the accelerator path on @menu_item, through which runtime changes of the
  * menu item's accelerator caused by the user can be identified and saved to
@@ -1126,6 +1147,9 @@ _gtk_menu_item_refresh_accel_path (GtkMenuItem   *menu_item,
  * This function is basically a convenience wrapper that handles calling
  * gtk_widget_set_accel_path() with the appropriate accelerator group for
  * the menu item.
+ *
+ * Note that you do need to set an accelerator on the parent menu with
+ * gtk_menu_set_accel_group() for this to work.
  */
 void
 gtk_menu_item_set_accel_path (GtkMenuItem *menu_item,
@@ -1134,7 +1158,8 @@ gtk_menu_item_set_accel_path (GtkMenuItem *menu_item,
   GtkWidget *widget;
 
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
-  g_return_if_fail (accel_path && accel_path[0] == '<' && strchr (accel_path, '/'));
+  g_return_if_fail (accel_path == NULL ||
+		    (accel_path[0] == '<' && strchr (accel_path, '/')));
 
   widget = GTK_WIDGET (menu_item);
 
