@@ -30,9 +30,9 @@
 #include "gtkbindings.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
+#include "gtkmenubar.h"
 #include "gtkmenuitem.h"
 #include "gtkmenushell.h"
-#include "gtksignal.h"
 #include "gtktearoffmenuitem.h"
 #include "gtkwindow.h"
 
@@ -44,6 +44,7 @@ enum {
   MOVE_CURRENT,
   ACTIVATE_CURRENT,
   CANCEL,
+  CYCLE_FOCUS,
   LAST_SIGNAL
 };
 
@@ -138,7 +139,7 @@ static gint gtk_menu_shell_is_item           (GtkMenuShell      *menu_shell,
 					      GtkWidget         *child);
 static GtkWidget *gtk_menu_shell_get_item    (GtkMenuShell      *menu_shell,
 					      GdkEvent          *event);
-static GtkType    gtk_menu_shell_child_type  (GtkContainer      *container);
+static GType    gtk_menu_shell_child_type  (GtkContainer      *container);
 static void gtk_menu_shell_real_select_item  (GtkMenuShell      *menu_shell,
 					      GtkWidget         *menu_item);
 static void gtk_menu_shell_select_submenu_first (GtkMenuShell   *menu_shell); 
@@ -148,50 +149,85 @@ static void gtk_real_menu_shell_move_current (GtkMenuShell      *menu_shell,
 static void gtk_real_menu_shell_activate_current (GtkMenuShell      *menu_shell,
 						  gboolean           force_hide);
 static void gtk_real_menu_shell_cancel           (GtkMenuShell      *menu_shell);
+static void gtk_real_menu_shell_cycle_focus      (GtkMenuShell      *menu_shell,
+						  GtkDirectionType   dir);
 
 static GtkContainerClass *parent_class = NULL;
 static guint menu_shell_signals[LAST_SIGNAL] = { 0 };
 
 
-GtkType
+GType
 gtk_menu_shell_get_type (void)
 {
-  static GtkType menu_shell_type = 0;
+  static GType menu_shell_type = 0;
 
   if (!menu_shell_type)
     {
-      static const GtkTypeInfo menu_shell_info =
+      static const GTypeInfo menu_shell_info =
       {
-	"GtkMenuShell",
-	sizeof (GtkMenuShell),
 	sizeof (GtkMenuShellClass),
-	(GtkClassInitFunc) gtk_menu_shell_class_init,
-	(GtkObjectInitFunc) gtk_menu_shell_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_menu_shell_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkMenuShell),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_menu_shell_init,
+	NULL,		/* value_table */
       };
 
-      menu_shell_type = gtk_type_unique (gtk_container_get_type (), &menu_shell_info);
+      menu_shell_type =
+	g_type_register_static (GTK_TYPE_CONTAINER, "GtkMenuShell",
+				&menu_shell_info, G_TYPE_FLAG_ABSTRACT);
     }
 
   return menu_shell_type;
 }
 
+static guint
+binding_signal_new (const gchar	       *signal_name,
+		    GType		itype,
+		    GSignalFlags	signal_flags,
+		    GCallback           handler,
+		    GSignalAccumulator  accumulator,
+		    gpointer		accu_data,
+		    GSignalCMarshaller  c_marshaller,
+		    GType		return_type,
+		    guint		n_params,
+		    ...)
+{
+  va_list args;
+  guint signal_id;
+
+  g_return_val_if_fail (signal_name != NULL, 0);
+  
+  va_start (args, n_params);
+
+  signal_id = g_signal_new_valist (signal_name, itype, signal_flags,
+                                   g_cclosure_new (handler, NULL, NULL),
+				   accumulator, accu_data, c_marshaller,
+                                   return_type, n_params, args);
+
+  va_end (args);
+ 
+  return signal_id;
+}
+
 static void
 gtk_menu_shell_class_init (GtkMenuShellClass *klass)
 {
-  GtkObjectClass *object_class;
+  GObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
 
   GtkBindingSet *binding_set;
 
-  object_class = (GtkObjectClass*) klass;
+  object_class = (GObjectClass*) klass;
   widget_class = (GtkWidgetClass*) klass;
   container_class = (GtkContainerClass*) klass;
 
-  parent_class = gtk_type_class (gtk_container_get_type ());
+  parent_class = g_type_class_peek_parent (klass);
 
   widget_class->realize = gtk_menu_shell_realize;
   widget_class->button_press_event = gtk_menu_shell_button_press;
@@ -215,42 +251,57 @@ gtk_menu_shell_class_init (GtkMenuShellClass *klass)
   klass->insert = gtk_menu_shell_real_insert;
 
   menu_shell_signals[DEACTIVATE] =
-    gtk_signal_new ("deactivate",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuShellClass, deactivate),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("deactivate",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuShellClass, deactivate),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   menu_shell_signals[SELECTION_DONE] =
-    gtk_signal_new ("selection-done",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuShellClass, selection_done),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("selection-done",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuShellClass, selection_done),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   menu_shell_signals[MOVE_CURRENT] =
-    gtk_signal_new ("move_current",
-		    GTK_RUN_LAST | GTK_RUN_ACTION,
-		    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuShellClass, move_current),
-		    _gtk_marshal_VOID__ENUM,
-		    GTK_TYPE_NONE, 1, 
-		    GTK_TYPE_MENU_DIRECTION_TYPE);
+    g_signal_new ("move_current",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkMenuShellClass, move_current),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__ENUM,
+		  G_TYPE_NONE, 1, 
+		  GTK_TYPE_MENU_DIRECTION_TYPE);
   menu_shell_signals[ACTIVATE_CURRENT] =
-    gtk_signal_new ("activate_current",
-		    GTK_RUN_LAST | GTK_RUN_ACTION,
-		    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuShellClass, activate_current),
-		    _gtk_marshal_VOID__BOOLEAN,
-		    GTK_TYPE_NONE, 1, 
-		    GTK_TYPE_BOOL);
+    g_signal_new ("activate_current",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkMenuShellClass, activate_current),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__BOOLEAN,
+		  G_TYPE_NONE, 1, 
+		  G_TYPE_BOOLEAN);
   menu_shell_signals[CANCEL] =
-    gtk_signal_new ("cancel",
-		    GTK_RUN_LAST | GTK_RUN_ACTION,
-		    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuShellClass, cancel),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("cancel",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkMenuShellClass, cancel),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
+  menu_shell_signals[CYCLE_FOCUS] =
+    binding_signal_new ("cycle_focus",
+			G_OBJECT_CLASS_TYPE (object_class),
+			G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			G_CALLBACK (gtk_real_menu_shell_cycle_focus),
+			NULL, NULL,
+			_gtk_marshal_VOID__ENUM,
+			G_TYPE_NONE, 1,
+			GTK_TYPE_DIRECTION_TYPE);
+
 
   binding_set = gtk_binding_set_by_class (klass);
   gtk_binding_entry_add_signal (binding_set,
@@ -259,26 +310,34 @@ gtk_menu_shell_class_init (GtkMenuShellClass *klass)
   gtk_binding_entry_add_signal (binding_set,
 				GDK_Return, 0,
 				"activate_current", 1,
-				GTK_TYPE_BOOL,
+				G_TYPE_BOOLEAN,
 				TRUE);
   gtk_binding_entry_add_signal (binding_set,
 				GDK_KP_Enter, 0,
 				"activate_current", 1,
-				GTK_TYPE_BOOL,
+				G_TYPE_BOOLEAN,
 				TRUE);
   gtk_binding_entry_add_signal (binding_set,
 				GDK_space, 0,
 				"activate_current", 1,
-				GTK_TYPE_BOOL,
+				G_TYPE_BOOLEAN,
 				FALSE);
   gtk_binding_entry_add_signal (binding_set,
 				GDK_KP_Space, 0,
 				"activate_current", 1,
-				GTK_TYPE_BOOL,
+				G_TYPE_BOOLEAN,
 				FALSE);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_F10, 0,
+				"cycle_focus", 1,
+                                GTK_TYPE_DIRECTION_TYPE, GTK_DIR_TAB_FORWARD);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_F10, GDK_SHIFT_MASK,
+				"cycle_focus", 1,
+                                GTK_TYPE_DIRECTION_TYPE, GTK_DIR_TAB_BACKWARD);
 }
 
-static GtkType
+static GType
 gtk_menu_shell_child_type (GtkContainer     *container)
 {
   return GTK_TYPE_MENU_ITEM;
@@ -344,7 +403,7 @@ gtk_menu_shell_deactivate (GtkMenuShell *menu_shell)
 {
   g_return_if_fail (GTK_IS_MENU_SHELL (menu_shell));
 
-  gtk_signal_emit (GTK_OBJECT (menu_shell), menu_shell_signals[DEACTIVATE]);
+  g_signal_emit (menu_shell, menu_shell_signals[DEACTIVATE], 0);
 }
 
 static void
@@ -381,6 +440,17 @@ gtk_menu_shell_realize (GtkWidget *widget)
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
 }
 
+void
+_gtk_menu_shell_activate (GtkMenuShell *menu_shell)
+{
+  if (!menu_shell->active)
+    {
+      gtk_grab_add (GTK_WIDGET (menu_shell));
+      menu_shell->have_grab = TRUE;
+      menu_shell->active = TRUE;
+    }
+}
+
 static gint
 gtk_menu_shell_button_press (GtkWidget      *widget,
 			     GdkEventButton *event)
@@ -402,12 +472,8 @@ gtk_menu_shell_button_press (GtkWidget      *widget,
     }
   else if (!menu_shell->active || !menu_shell->button)
     {
-      if (!menu_shell->active)
-	{
-	  gtk_grab_add (GTK_WIDGET (widget));
-	  menu_shell->have_grab = TRUE;
-	  menu_shell->active = TRUE;
-	}
+      _gtk_menu_shell_activate (menu_shell);
+      
       menu_shell->button = event->button;
 
       menu_item = gtk_menu_shell_get_item (menu_shell, (GdkEvent *)event);
@@ -431,7 +497,7 @@ gtk_menu_shell_button_press (GtkWidget      *widget,
       if (widget == GTK_WIDGET (menu_shell))
 	{
 	  gtk_menu_shell_deactivate (menu_shell);
-	  gtk_signal_emit (GTK_OBJECT (menu_shell), menu_shell_signals[SELECTION_DONE]);
+	  g_signal_emit (menu_shell, menu_shell_signals[SELECTION_DONE], 0);
 	}
     }
 
@@ -529,7 +595,7 @@ gtk_menu_shell_button_release (GtkWidget      *widget,
       if (deactivate)
 	{
 	  gtk_menu_shell_deactivate (menu_shell);
-	  gtk_signal_emit (GTK_OBJECT (menu_shell), menu_shell_signals[SELECTION_DONE]);
+	  g_signal_emit (menu_shell, menu_shell_signals[SELECTION_DONE], 0);
 	}
     }
 
@@ -558,7 +624,7 @@ gtk_menu_shell_key_press (GtkWidget	*widget,
   if (GTK_IS_WINDOW (toplevel) &&
       _gtk_window_activate_key (GTK_WINDOW (toplevel), event))
     return TRUE;
-  
+
   return FALSE;
 }
 
@@ -736,9 +802,11 @@ gtk_real_menu_shell_deactivate (GtkMenuShell *menu_shell)
 	}
       if (menu_shell->have_xgrab)
 	{
+	  GdkDisplay *display = gtk_widget_get_display (GTK_WIDGET (menu_shell));
+	  
 	  menu_shell->have_xgrab = FALSE;
-	  gdk_pointer_ungrab (GDK_CURRENT_TIME);
-	  gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+	  gdk_display_pointer_ungrab (display, GDK_CURRENT_TIME);
+	  gdk_display_keyboard_ungrab (display, GDK_CURRENT_TIME);
 	}
     }
 }
@@ -806,6 +874,9 @@ gtk_menu_shell_real_select_item (GtkMenuShell *menu_shell,
 {
   gtk_menu_shell_deselect (menu_shell);
 
+  if (!_gtk_menu_item_is_selectable (menu_item))
+    return;
+
   menu_shell->active_menu_item = menu_item;
   _gtk_menu_item_set_placement (GTK_MENU_ITEM (menu_shell->active_menu_item),
 			       GTK_MENU_SHELL_GET_CLASS (menu_shell)->submenu_placement);
@@ -844,7 +915,7 @@ gtk_menu_shell_activate_item (GtkMenuShell      *menu_shell,
   if (!deactivate)
     deactivate = GTK_MENU_ITEM_GET_CLASS (menu_item)->hide_on_activate;
 
-  gtk_widget_ref (GTK_WIDGET (menu_shell));
+  g_object_ref (menu_shell);
 
   if (deactivate)
     {
@@ -852,7 +923,7 @@ gtk_menu_shell_activate_item (GtkMenuShell      *menu_shell,
 
       do
 	{
-	  gtk_widget_ref (GTK_WIDGET (parent_menu_shell));
+	  g_object_ref (parent_menu_shell);
 	  shells = g_slist_prepend (shells, parent_menu_shell);
 	  parent_menu_shell = (GtkMenuShell*) parent_menu_shell->parent_menu_shell;
 	}
@@ -864,19 +935,19 @@ gtk_menu_shell_activate_item (GtkMenuShell      *menu_shell,
       /* flush the x-queue, so any grabs are removed and
        * the menu is actually taken down
        */
-      gdk_flush ();
+      gdk_display_sync (gtk_widget_get_display (menu_item));
     }
 
   gtk_widget_activate (menu_item);
 
   for (slist = shells; slist; slist = slist->next)
     {
-      gtk_signal_emit (slist->data, menu_shell_signals[SELECTION_DONE]);
-      gtk_widget_unref (slist->data);
+      g_signal_emit (slist->data, menu_shell_signals[SELECTION_DONE], 0);
+      g_object_unref (slist->data);
     }
   g_slist_free (shells);
 
-  gtk_widget_unref (GTK_WIDGET (menu_shell));
+  g_object_unref (menu_shell);
 }
 
 /* Distance should be +/- 1 */
@@ -921,14 +992,21 @@ gtk_menu_shell_move_selected (GtkMenuShell  *menu_shell,
 }
 
 /**
- * _gtk_menu_shell_select_first:
+ * gtk_menu_shell_select_first:
  * @menu_shell: a #GtkMenuShell
+ * @search_sensitive: if %TRUE, search for the first selectable
+ *                    menu item, otherwise select nothing if
+ *                    the first item isn't sensitive. This
+ *                    should be %FALSE if the menu is being
+ *                    popped up initially.
  * 
- * Select the first visible child of the menu shell, unless
- * it's a tearoff item.
+ * Select the first visible or selectable child of the menu shell;
+ * don't select tearoff items unless the only item is a tearoff
+ * item.
  **/
 void
-_gtk_menu_shell_select_first (GtkMenuShell *menu_shell)
+gtk_menu_shell_select_first (GtkMenuShell *menu_shell,
+			     gboolean      search_sensitive)
 {
   GtkWidget *to_select = NULL;
   GList *tmp_list;
@@ -938,7 +1016,8 @@ _gtk_menu_shell_select_first (GtkMenuShell *menu_shell)
     {
       GtkWidget *child = tmp_list->data;
       
-      if (GTK_WIDGET_VISIBLE (child))
+      if ((!search_sensitive && GTK_WIDGET_VISIBLE (child)) ||
+	  _gtk_menu_item_is_selectable (child))
 	{
 	  to_select = child;
 	  if (!GTK_IS_TEAROFF_MENU_ITEM (child))
@@ -952,6 +1031,33 @@ _gtk_menu_shell_select_first (GtkMenuShell *menu_shell)
     gtk_menu_shell_select_item (menu_shell, to_select);
 }
 
+static void
+gtk_menu_shell_select_last (GtkMenuShell *menu_shell,
+			    gboolean      search_sensitive)
+{
+  GtkWidget *to_select = NULL;
+  GList *tmp_list;
+
+  tmp_list = g_list_last (menu_shell->children);
+  while (tmp_list)
+    {
+      GtkWidget *child = tmp_list->data;
+      
+      if ((!search_sensitive && GTK_WIDGET_VISIBLE (child)) ||
+	  _gtk_menu_item_is_selectable (child))
+	{
+	  to_select = child;
+	  if (!GTK_IS_TEAROFF_MENU_ITEM (child))
+	    break;
+	}
+      
+      tmp_list = tmp_list->prev;
+    }
+
+  if (to_select)
+    gtk_menu_shell_select_item (menu_shell, to_select);
+}
+
 static void
 gtk_menu_shell_select_submenu_first (GtkMenuShell     *menu_shell)
 {
@@ -960,7 +1066,7 @@ gtk_menu_shell_select_submenu_first (GtkMenuShell     *menu_shell)
   menu_item = GTK_MENU_ITEM (menu_shell->active_menu_item); 
   
   if (menu_item->submenu)
-    _gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu));
+    gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu), TRUE);
 }
 
 static void
@@ -1026,14 +1132,14 @@ gtk_real_menu_shell_move_current (GtkMenuShell      *menu_shell,
       if (!had_selection &&
 	  !menu_shell->active_menu_item &&
 	  menu_shell->children)
-	gtk_menu_shell_select_item (menu_shell, g_list_last (menu_shell->children)->data);
+	gtk_menu_shell_select_last (menu_shell, TRUE);
       break;
     case GTK_MENU_DIR_NEXT:
       gtk_menu_shell_move_selected (menu_shell, 1);
       if (!had_selection &&
 	  !menu_shell->active_menu_item &&
 	  menu_shell->children)
-	gtk_menu_shell_select_item (menu_shell, menu_shell->children->data);
+	gtk_menu_shell_select_first (menu_shell, TRUE);
       break;
     }
   
@@ -1061,5 +1167,21 @@ gtk_real_menu_shell_cancel (GtkMenuShell      *menu_shell)
   gtk_menu_shell_deselect (menu_shell);
   
   gtk_menu_shell_deactivate (menu_shell);
-  gtk_signal_emit (GTK_OBJECT (menu_shell), menu_shell_signals[SELECTION_DONE]);
+  g_signal_emit (menu_shell, menu_shell_signals[SELECTION_DONE], 0);
+}
+
+static void
+gtk_real_menu_shell_cycle_focus (GtkMenuShell      *menu_shell,
+				 GtkDirectionType   dir)
+{
+  while (menu_shell && !GTK_IS_MENU_BAR (menu_shell))
+    {
+      if (menu_shell->parent_menu_shell)
+	menu_shell = GTK_MENU_SHELL (menu_shell->parent_menu_shell);
+      else
+	menu_shell = NULL;
+    }
+
+  if (menu_shell)
+    _gtk_menu_bar_cycle_focus (GTK_MENU_BAR (menu_shell), dir);
 }
