@@ -26,7 +26,15 @@
 #include "gtktreeview.h"
 #include "gtktreeprivate.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
+
+
+#define INITIALIZE_TREE_ITER(Iter) \
+    G_STMT_START{ \
+      (Iter)->stamp = 0; \
+      (Iter)->user_data  = NULL; \
+      (Iter)->user_data2 = NULL; \
+      (Iter)->user_data3 = NULL; \
+    }G_STMT_END
 
 
 struct _GtkTreePath
@@ -38,10 +46,10 @@ struct _GtkTreePath
 static void gtk_tree_model_base_init (gpointer g_class);
 
 
-GtkType
+GType
 gtk_tree_model_get_type (void)
 {
-  static GtkType tree_model_type = 0;
+  static GType tree_model_type = 0;
 
   if (! tree_model_type)
     {
@@ -58,7 +66,10 @@ gtk_tree_model_get_type (void)
 	NULL
       };
 
-      tree_model_type = g_type_register_static (G_TYPE_INTERFACE, "GtkTreeModel", &tree_model_info, 0);
+      tree_model_type =
+	g_type_register_static (G_TYPE_INTERFACE, "GtkTreeModel",
+				&tree_model_info, 0);
+
       g_type_interface_add_prerequisite (tree_model_type, G_TYPE_OBJECT);
     }
 
@@ -124,7 +135,7 @@ gtk_tree_model_base_init (gpointer g_class)
 /**
  * gtk_tree_path_new:
  *
- * Creates a new #GtkTreePath.  This structure refers to a row 
+ * Creates a new #GtkTreePath.  This structure refers to a row.
  *
  * Return value: A newly created #GtkTreePath.
  **/
@@ -190,6 +201,39 @@ gtk_tree_path_new_from_string (const gchar *path)
   return retval;
 }
 
+/**
+ * gtk_tree_path_new_from_indices:
+ * @first_index: first integer
+ * @varargs: list of integers terminated by -1
+ *
+ * Creates a new path with @first_index and @varargs as indices.
+ *
+ * Return value: A newly created GtkTreePath.
+ **/
+GtkTreePath *
+gtk_tree_path_new_from_indices (gint first_index,
+				...)
+{
+  int arg;
+  va_list args;
+  GtkTreePath *path;
+
+  path = gtk_tree_path_new ();
+
+  va_start (args, first_index);
+  arg = first_index;
+
+  while (arg != -1)
+    {
+      gtk_tree_path_append_index (path, arg);
+      arg = va_arg (args, gint);
+    }
+
+  va_end (args);
+
+  return path;
+}
+
 /**
  * gtk_tree_path_to_string:
  * @path: A #GtkTreePath
@@ -275,7 +319,11 @@ void
 gtk_tree_path_prepend_index (GtkTreePath *path,
 			     gint       index)
 {
-  gint *new_indices = g_new (gint, ++path->depth);
+  gint *new_indices;
+
+  (path->depth)++;
+  new_indices = g_new (gint, path->depth);
+
   if (path->indices == NULL)
     {
       path->indices = new_indices;
@@ -345,7 +393,7 @@ gtk_tree_path_free (GtkTreePath *path)
  * Return value: A new #GtkTreePath.
  **/
 GtkTreePath *
-gtk_tree_path_copy (GtkTreePath *path)
+gtk_tree_path_copy (const GtkTreePath *path)
 {
   GtkTreePath *retval;
 
@@ -358,6 +406,19 @@ gtk_tree_path_copy (GtkTreePath *path)
   return retval;
 }
 
+GType
+gtk_tree_path_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("GtkTreePath",
+					     (GBoxedCopyFunc) gtk_tree_path_copy,
+					     (GBoxedFreeFunc) gtk_tree_path_free);
+
+  return our_type;
+}
+
 /**
  * gtk_tree_path_compare:
  * @a: A #GtkTreePath.
@@ -569,6 +630,19 @@ gtk_tree_iter_free (GtkTreeIter *iter)
   g_free (iter);
 }
 
+GType
+gtk_tree_iter_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("GtkTreeIter",
+					     (GBoxedCopyFunc) gtk_tree_iter_copy,
+					     (GBoxedFreeFunc) gtk_tree_iter_free);
+
+  return our_type;
+}
+
 /**
  * gtk_tree_model_get_flags:
  * @tree_model: A #GtkTreeModel.
@@ -648,6 +722,8 @@ gtk_tree_model_get_iter (GtkTreeModel *tree_model,
   g_return_val_if_fail (GTK_TREE_MODEL_GET_IFACE (tree_model)->get_iter != NULL, FALSE);
   g_return_val_if_fail (path->depth > 0, FALSE);
 
+  INITIALIZE_TREE_ITER (iter);
+
   return (* GTK_TREE_MODEL_GET_IFACE (tree_model)->get_iter) (tree_model, iter, path);
 }
 
@@ -684,6 +760,36 @@ gtk_tree_model_get_iter_from_string (GtkTreeModel *tree_model,
   return retval;
 }
 
+/**
+ * gtk_tree_model_get_string_from_iter:
+ * @tree_model: A #GtkTreeModel.
+ * @iter: An #GtkTreeIter.
+ *
+ * Generates a string representation of the iter. This string is a ':'
+ * separated list of numbers. For example, "4:10:0:3" would be an
+ * acceptable return value for this string.
+ *
+ * Return value: A newly-allocated string. Must be freed with g_free().
+ **/
+gchar *
+gtk_tree_model_get_string_from_iter (GtkTreeModel *tree_model,
+                                     GtkTreeIter  *iter)
+{
+  GtkTreePath *path;
+  gchar *ret;
+
+  g_return_val_if_fail (GTK_IS_TREE_MODEL (tree_model), NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+
+  path = gtk_tree_model_get_path (tree_model, iter);
+
+  g_return_val_if_fail (path != NULL, NULL);
+
+  ret = gtk_tree_path_to_string (path);
+  gtk_tree_path_free (path);
+
+  return ret;
+}
 
 /**
  * gtk_tree_model_get_iter_first:
@@ -782,12 +888,15 @@ gtk_tree_model_iter_next (GtkTreeModel  *tree_model,
  * gtk_tree_model_iter_children:
  * @tree_model: A #GtkTreeModel.
  * @iter: The new #GtkTreeIter to be set to the child.
- * @parent: The #GtkTreeIter.
+ * @parent: The #GtkTreeIter, or %NULL
  *
  * Sets @iter to point to the first child of @parent.  If @parent has no children,
  * %FALSE is returned and @iter is set to be invalid.  @parent will remain a valid
  * node after this function has been called.
  *
+ * If @parent is %NULL returns the first node, equivalent to
+ * <literal>gtk_tree_model_get_iter_first (tree_model, iter);</literal>
+ *
  * Return value: %TRUE, if @child has been set to the first child.
  **/
 gboolean
@@ -799,6 +908,8 @@ gtk_tree_model_iter_children (GtkTreeModel *tree_model,
   g_return_val_if_fail (iter != NULL, FALSE);
   g_return_val_if_fail (GTK_TREE_MODEL_GET_IFACE (tree_model)->iter_children != NULL, FALSE);
 
+  INITIALIZE_TREE_ITER (iter);
+
   return (* GTK_TREE_MODEL_GET_IFACE (tree_model)->iter_children) (tree_model, iter, parent);
 }
 
@@ -853,9 +964,9 @@ gtk_tree_model_iter_n_children (GtkTreeModel *tree_model,
  * index is 0.  If @index is too big, or @parent has no children, @iter is set
  * to an invalid iterator and %FALSE is returned.  @parent will remain a valid
  * node after this function has been called.  As a special case, if @parent is
- * %NULL, then the @n<!>th root node is set.
+ * %NULL, then the @n<!-- -->th root node is set.
  *
- * Return value: %TRUE, if @parent has an @n<!>th child.
+ * Return value: %TRUE, if @parent has an @n<!-- -->th child.
  **/
 gboolean
 gtk_tree_model_iter_nth_child (GtkTreeModel *tree_model,
@@ -868,6 +979,8 @@ gtk_tree_model_iter_nth_child (GtkTreeModel *tree_model,
   g_return_val_if_fail (n >= 0, FALSE);
   g_return_val_if_fail (GTK_TREE_MODEL_GET_IFACE (tree_model)->iter_nth_child != NULL, FALSE);
 
+  INITIALIZE_TREE_ITER (iter);
+
   return (* GTK_TREE_MODEL_GET_IFACE (tree_model)->iter_nth_child) (tree_model, iter, parent, n);
 }
 
@@ -894,6 +1007,8 @@ gtk_tree_model_iter_parent (GtkTreeModel *tree_model,
   g_return_val_if_fail (child != NULL, FALSE);
   g_return_val_if_fail (GTK_TREE_MODEL_GET_IFACE (tree_model)->iter_parent != NULL, FALSE);
 
+  INITIALIZE_TREE_ITER (iter);
+
   return (* GTK_TREE_MODEL_GET_IFACE (tree_model)->iter_parent) (tree_model, iter, child);
 }
 
@@ -1145,26 +1260,25 @@ gtk_tree_model_foreach_helper (GtkTreeModel            *model,
 			       GtkTreeModelForeachFunc  func,
 			       gpointer                 user_data)
 {
-  gtk_tree_path_append_index (path, 0);
-
   do
     {
       GtkTreeIter child;
 
-      if (gtk_tree_model_iter_children (model, &child, iter))
-	{
-	  if (gtk_tree_model_foreach_helper (model, &child, path, func, user_data))
-	    return TRUE;
-	}
-
       if ((* func) (model, path, iter, user_data))
 	return TRUE;
 
+      if (gtk_tree_model_iter_children (model, &child, iter))
+	{
+	  gtk_tree_path_down (path);
+	  if (gtk_tree_model_foreach_helper (model, &child, path, func, user_data))
+	    return TRUE;
+	  gtk_tree_path_up (path);
+	}
+
       gtk_tree_path_next (path);
     }
   while (gtk_tree_model_iter_next (model, iter));
 
-  gtk_tree_path_up (path);
   return FALSE;
 }
 
@@ -1210,6 +1324,20 @@ static void gtk_tree_row_reference_unref_path (GtkTreePath  *path,
 					       gint          depth);
 
 
+GType
+gtk_tree_row_reference_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("GtkTreeRowReference",
+					     (GBoxedCopyFunc) gtk_tree_row_reference_copy,
+					     (GBoxedFreeFunc) gtk_tree_row_reference_free);
+
+  return our_type;
+}
+
+
 #define ROW_REF_DATA_STRING "gtk-tree-row-refs"
 
 struct _GtkTreeRowReference
@@ -1231,6 +1359,9 @@ release_row_references (gpointer data)
   RowRefList *refs = data;
   GSList *tmp_list = NULL;
 
+  if (!refs->list)
+    return;
+
   tmp_list = refs->list;
   while (tmp_list != NULL)
     {
@@ -1343,7 +1474,14 @@ gtk_tree_row_ref_deleted_callback (GObject     *object,
 	  /* We know it affects us. */
 	  if (path->indices[i] == reference->path->indices[i])
 	    {
-	      gtk_tree_row_reference_unref_path (reference->path, reference->model, reference->path->depth - 1);
+	      if (reference->path->depth > path->depth)
+		/* some parent was deleted, trying to unref any node
+		 * between the deleted parent and the node the reference
+		 * is pointing to is bad, as those nodes are already gone.
+		 */
+		gtk_tree_row_reference_unref_path (reference->path, reference->model, path->depth - 1);
+	      else
+	        gtk_tree_row_reference_unref_path (reference->path, reference->model, reference->path->depth - 1);
 	      gtk_tree_path_free (reference->path);
 	      reference->path = NULL;
 	    }
@@ -1413,17 +1551,17 @@ gtk_tree_row_ref_reordered_callback (GObject     *object,
 static void
 connect_ref_callbacks (GtkTreeModel *model)
 {
-  g_signal_connect (G_OBJECT (model),
+  g_signal_connect (model,
 		    "row_inserted",
-		    (GCallback) gtk_tree_row_ref_inserted_callback,
+		    G_CALLBACK (gtk_tree_row_ref_inserted_callback),
 		    model);
-  g_signal_connect (G_OBJECT (model),
+  g_signal_connect (model,
 		    "row_deleted",
-		    (GCallback) gtk_tree_row_ref_deleted_callback,
+		    G_CALLBACK (gtk_tree_row_ref_deleted_callback),
 		    model);
-  g_signal_connect (G_OBJECT (model),
+  g_signal_connect (model,
 		    "rows_reordered",
-		    (GCallback) gtk_tree_row_ref_reordered_callback,
+		    G_CALLBACK (gtk_tree_row_ref_reordered_callback),
 		    model);
 }
 
@@ -1464,17 +1602,17 @@ gtk_tree_row_reference_unref_path (GtkTreePath  *path,
 static void
 disconnect_ref_callbacks (GtkTreeModel *model)
 {
-  g_signal_handlers_disconnect_matched (G_OBJECT (model),
+  g_signal_handlers_disconnect_matched (model,
                                         G_SIGNAL_MATCH_FUNC,
                                         0, 0, NULL,
 					gtk_tree_row_ref_inserted_callback,
 					NULL);
-  g_signal_handlers_disconnect_matched (G_OBJECT (model),
+  g_signal_handlers_disconnect_matched (model,
                                         G_SIGNAL_MATCH_FUNC,
                                         0, 0, NULL,
 					gtk_tree_row_ref_deleted_callback,
 					NULL);
-  g_signal_handlers_disconnect_matched (G_OBJECT (model),
+  g_signal_handlers_disconnect_matched (model,
                                         G_SIGNAL_MATCH_FUNC,
                                         0, 0, NULL,
 					gtk_tree_row_ref_reordered_callback,
@@ -1626,6 +1764,15 @@ gtk_tree_row_reference_valid (GtkTreeRowReference *reference)
   return TRUE;
 }
 
+
+GtkTreeRowReference *
+gtk_tree_row_reference_copy (GtkTreeRowReference *reference)
+{
+  return gtk_tree_row_reference_new_proxy (reference->proxy,
+					   reference->model,
+					   reference->path);
+}
+
 /**
  * gtk_tree_row_reference_free:
  * @reference: A #GtkTreeRowReference, or NULL
@@ -1652,7 +1799,9 @@ gtk_tree_row_reference_free (GtkTreeRowReference *reference)
 
   if (refs->list == NULL)
     {
-      disconnect_ref_callbacks (reference->model);
+      if (G_OBJECT (reference->model) == reference->proxy)
+	disconnect_ref_callbacks (reference->model);
+
       g_object_set_data (G_OBJECT (reference->proxy),
 			 ROW_REF_DATA_STRING,
 			 NULL);
@@ -1688,7 +1837,7 @@ gtk_tree_row_reference_inserted (GObject     *proxy,
 }
 
 /**
- * gtk_tree_row_reference_inserted:
+ * gtk_tree_row_reference_deleted:
  * @proxy: A #GObject
  * @path: The path position that was deleted
  * 
